///////////////////////////////////////////////////////////////////////////////
//
// Mesh Settings Dialog
// Copyright (C) 2006 Tobias Minch
//
// Part of rct3tool
// Copyright 2005 Jonathan Wilson
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Written by
//   Tobias Minich - belgabor@gmx.de
//
// Parts of this code are based on code written for rct3tool (Copyright 2005
// Jonathan Wilson).
//
///////////////////////////////////////////////////////////////////////////////

#include "wx_pch.h"

#include <wx/valgen.h>

#include "SCNFile.h"

#include "wxdlgMesh.h"

BEGIN_EVENT_TABLE(dlgMesh,wxDialog)
EVT_CHOICE(XRCID("m_choiceTextureFlags"), dlgMesh::OnChange)
EVT_CHECKBOX(XRCID("m_checkDontImport"), dlgMesh::OnChange)
EVT_BUTTON(XRCID("m_btLoad"), dlgMesh::OnLoad)
END_EVENT_TABLE()

dlgMesh::dlgMesh(const wxArrayString *textures, wxWindow *parent) {
    InitWidgetsFromXRC((wxWindow *)parent);

    m_ms.unknown = 3;
    m_unknown = 1;
    m_applyall = false;

    if (textures)
        m_cbTextureName->Append(*textures);

    m_checkDontImport->SetValidator(wxGenericValidator(&m_ms.disabled));
    m_cbTextureName->SetValidator(wxGenericValidator(&m_ms.FTX));
    m_choiceTextureStyle->SetValidator(wxGenericValidator(&m_ms.TXS));
    m_choiceTextureFlags->SetValidator(wxGenericValidator(&m_flags));
    m_choiceShowTexture->SetValidator(wxGenericValidator(&m_place));
    m_choiceUnknown->SetValidator(wxGenericValidator(&m_unknown));
    m_checkApplyAll->SetValidator(wxGenericValidator(&m_applyall));

    m_btOk->SetId(wxID_OK);
    m_btCancel->SetId(wxID_CANCEL);
}

void dlgMesh::RefreshAll() {
    TransferDataFromWindow();
    m_choiceTextureFlags->Enable(!m_ms.disabled);
    m_choiceShowTexture->Enable(!m_ms.disabled);
    m_choiceUnknown->Enable(!m_ms.disabled);
    if (m_ms.disabled) {
        m_cbTextureName->Enable(false);
        m_choiceTextureStyle->Enable(false);
        return;
    }
    bool do_enable = true;
    switch (m_flags) {
    case 1:
        m_choiceTextureStyle->SetStringSelection(wxT("SIOpaqueUnlit"));
        break;
    case 2:
        do_enable = false;
        m_cbTextureName->SetValue(wxT("useterraintexture"));
        m_choiceTextureStyle->SetStringSelection(wxT("SIOpaque"));
        break;
    case 3:
        do_enable = false;
        m_cbTextureName->SetValue(wxT("useclifftexture"));
        m_choiceTextureStyle->SetStringSelection(wxT("SIOpaque"));
        break;
    case 4:
        do_enable = false;
        m_cbTextureName->SetValue(wxT("siwater"));
        m_choiceTextureStyle->SetStringSelection(wxT("SIWater"));
        break;
    case 5:
        do_enable = false;
        m_cbTextureName->SetValue(wxT("watermask"));
        m_choiceTextureStyle->SetStringSelection(wxT("SIFillZ"));
        break;
    case 6:
        do_enable = false;
        m_cbTextureName->SetValue(wxT("UseAdTexture"));
        m_choiceTextureStyle->SetStringSelection(wxT("SIOpaque"));
        break;
    }
    m_cbTextureName->Enable(do_enable);
    m_choiceTextureStyle->Enable(do_enable);
}

void dlgMesh::SetMeshStruct(cMeshStruct ms) {
    m_ms = ms;

    m_place = ms.place;

    if (ms.flags == 0) {
        m_flags = 0;
    }
    if (ms.flags == 12) {
        m_flags = 1;
    }
    if (ms.flags == 12288) {
        m_flags = 2;
    }
    if (ms.flags == 20480) {
        m_flags = 3;
    }
    if (ms.flags == 32788) {
        m_flags = 6;
    }
    if (ms.FTX != wxT("")) {
        if (ms.FTX.CmpNoCase("siwater") == 0) {
            m_flags = 4;
        }
        if (ms.FTX.CmpNoCase("watermask") == 0) {
            m_flags = 5;
        }
    }
    if (ms.unknown == 1) {
        m_unknown = 0;
    } else {
        m_unknown = 1;
    }

    TransferDataToWindow();
    RefreshAll();
}

cMeshStruct dlgMesh::GetMeshStruct() {
    cMeshStruct ret = m_ms;

    ret.place = m_place;

    if (m_unknown == 0)
        ret.unknown = 1;
    else
        ret.unknown = 3;
    switch (m_flags) {
    case 1:
        ret.flags = 12;
        break;
    case 2:
        ret.flags = 12288;
        break;
    case 3:
        ret.flags = 20480;
        break;
    case 6:
        ret.flags = 32788;
        break;
    default:
        ret.flags = 0;
    }
    return ret;
}

void dlgMesh::OnLoad(wxCommandEvent& event) {
    wxFileDialog *dialog = new wxFileDialog(
                                     NULL,
                                     _T("Open Scenery File (Load mesh settings)"),
                                     wxEmptyString,
                                     wxEmptyString,
                                     _T("Scenery Files (*.scn)|*.scn"),
                                     wxFD_FILE_MUST_EXIST|wxFD_CHANGE_DIR,
                                     wxDefaultPosition,
                                     wxSize(600,400)
                                 );
    if (dialog->ShowModal() == wxID_OK) {
        cSCNFile *texscn = new cSCNFile(dialog->GetPath().fn_str());
        if (texscn) {
            if (texscn->meshstructs.size()==0) {
                wxMessageBox(_("The selected scenery file contains no meshes."), _("Warning"), wxOK | wxICON_WARNING, this);
                delete texscn;
                delete dialog;
                return;
            }
            wxArrayString choices;
            for (cMeshStructIterator ms = texscn->meshstructs.begin(); ms != texscn->meshstructs.end(); ms++) {
                choices.Add(ms->Name);
            }
            if (choices.size() > 0) {
                std::string title = "Choose mesh settings to import";
                int choice = ::wxGetSingleChoiceIndex(dialog->GetPath(), _("Choose mesh settings to import"), choices, this);
                if (choice >= 0) {
                    wxString name = m_ms.Name;
                    SetMeshStruct(texscn->meshstructs[choice]);
                    TransferDataToWindow();
                    RefreshAll();
                }
            }
            delete texscn;
        }
    }
    delete dialog;
}
