///////////////////////////////////////////////////////////////////////////////
//
// 3D Utility Library
// C++ structures for the importer
// Copyright (C) 2006 Tobias Minch
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Written by
//   Tobias Minich - belgabor@gmx.de
//
///////////////////////////////////////////////////////////////////////////////

#ifndef RCT3STRUCTS_H_INCLUDED
#define RCT3STRUCTS_H_INCLUDED

#include "wx_pch.h"

#include "sceneryvisual.h"
#include <vector>

typedef std::vector<wxString>::iterator cStringIterator;

struct cText {
	wxString name;
	wxString text;
};

struct cMeshStruct {
    bool disabled;
    wxString TXS;
    wxString FTX;
    unsigned long place;
    unsigned long flags;
    unsigned long unknown;
    wxString Name;
    bool effectpoint;
    D3DVECTOR effectpoint_vert;
};

typedef std::vector<cMeshStruct>::iterator cMeshStructIterator;

struct cFlexiTexture {
    wxString Name;
    wxString Texture;
    wxString Alpha;
    unsigned long Recolorable;
};

typedef std::vector<cFlexiTexture>::iterator cFlexiTextureIterator;

struct cEffectPoint {
	wxString Name;
	std::vector<D3DMATRIX> Transform;
	std::vector<wxString> TransformNames;
};

typedef std::vector<cEffectPoint>::iterator cEffectPointIterator;

class cSIVSettings {
public:
    unsigned int sivflags;
    float sway;
    float brightness;
    float unknown;
    float scale;

    cSIVSettings(bool as_tree = false) {
        if (as_tree) {
            sivflags = OVL_SIV_GREENERY | OVL_SIV_ROTATION;
            sway = 0.2;
            brightness = 0.8;
            unknown = 1.0;
            scale = 0.4;
        } else {
            sivflags = 0;
            sway = 0.0;
            brightness = 1.0;
            unknown = 1.0;
            scale = 0.0;
        }
    };
};

#endif // RCT3STRUCTS_H_INCLUDED
